var _interopRequireWildcard = function (obj) { return obj && obj.__esModule ? obj : { 'default': obj }; };

var _toConsumableArray = function (arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } };

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.findModules = findModules;

var _parse = require('acorn');

var _umd = require('acorn-umd');

var _umd2 = _interopRequireWildcard(_umd);

var _estraverse = require('estraverse');

var _estraverse2 = _interopRequireWildcard(_estraverse);

var _lodash$compact$includes$reject = require('lodash');

var _lodash$compact$includes$reject2 = _interopRequireWildcard(_lodash$compact$includes$reject);

var _updateReferences = require('./updateReferences');

var _updateReferences2 = _interopRequireWildcard(_updateReferences);

var _Error = require('./Error');

var _Error2 = _interopRequireWildcard(_Error);

var acornOptions = {
  ecmaVersion: 6,

  sourceType: 'module',

  // Be super loose (as who cares for this purpose)
  allowImportExportEverywhere: true,
  allowReturnOutsideFunction: true,
  allowHashBang: true
};

function findModules(path, _ref) {
  var imports = _ref.imports;
  var scope = _ref.scope;

  var result = [];
  _estraverse2['default'].traverse(scope, {
    enter: function enter(node) {
      switch (node.type) {
        case 'MemberExpression':
          if (_lodash$compact$includes$reject.includes(imports, node.object.name)) {
            if (node.computed) {
              var msg = 'Could not id computed function ' + node.object.name + '[' + node.property.name + ']';
              throw new _Error2['default'](msg, path);
            }
            result.push(node.property.name);
          }
          break;
        case 'CallExpression':
          // Detect chaining
          if (_lodash$compact$includes$reject.includes(imports, node.callee.name)) {
            result.push('chain');
          }
          break;
      }
    }
  });
  return result;
}

exports['default'] = function (code, path, options) {
  var ast = _parse.parse(code, _lodash$compact$includes$reject2['default'].assign({
    ranges: true,
    locations: true
  }, acornOptions, _lodash$compact$includes$reject2['default'].result(options, 'acorn')));

  var result = [];

  // imports to consider lodash (e.g. lodash-compact, lodash, etc)
  var lodashOptions = _lodash$compact$includes$reject.compact([options.lodash, options.output]);

  _lodash$compact$includes$reject2['default'](_umd2['default'](ast, {
    amd: false,
    cjs: _lodash$compact$includes$reject.includes(options.format, 'cjs'),
    es6: _lodash$compact$includes$reject.includes(options.format, 'es6')
  })).filter(function (node) {
    // consider adding lodash-fp & others
    return _lodash$compact$includes$reject.includes(lodashOptions, node.source.value);
  }).each(function (node) {
    // Add direct specifiers (`import {map, pick} from 'lodash'`)
    _lodash$compact$includes$reject2['default'](node.specifiers).map('imported').compact().each(function (requireNode) {
      result.push(requireNode.name);
    }).value();
  }).tap(function (nodes) {
    if (options.update && nodes.length) {
      _updateReferences2['default'](code, path, nodes, options);
    }
  }).map(function (node) {
    // filter the specifiers down to the direct imports
    // (handles `import x,{y,z} from 'foo';)
    return {
      imports: _lodash$compact$includes$reject.reject(node.specifiers, 'imported').map(function (x) {
        return x.local.name;
      }),
      scope: node.scope.block
    };
  }).each(function (node) {
    result.push.apply(result, _toConsumableArray(findModules(path, node)));
  }).value();

  if (typeof options.global === 'string') {
    result.push.apply(result, _toConsumableArray(findModules(path, {
      imports: [options.global],
      scope: ast
    })));
  }

  return result;
};